@@ -1,6 +1,6 @@
 /* lsmbox -- list nr of total and read messages for one or several mailboxes
  *
- * Copyright (C) 2002-2006, 2010 David Weinehall
+ * Copyright (C) 2002-2006, 2010, 2014 David Weinehall
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2
@@ -76,7 +76,8 @@ static void usage(void)
 		_("Usage: "), progname,
 		_(" [OPTION]... [MAILBOX]...\n"
 		  "List the number of total, old unread, and new messages\n"
-		  "for the specified mailbox(es).\n\n"),
+		  "for the specified mailbox(es).\n"
+		  "\n"),
 #ifdef USE_CURSES
 		_("  -c, --continuous      update statistics continuously\n"
 		  "  -i, --check-interval  interval between mailchecks\n"),
@@ -114,7 +115,7 @@ static void version(void)
 		PACKAGE_VERSION,
 		_("Written by David Weinehall.\n"
 		  "\n"
-		  "Copyright (C) 2002-2006 David Weinehall\n"
+		  "Copyright (C) 2002-2007, 2010, 2014 David Weinehall\n"
 		  "This is free software; see the source for copying "
 		  "conditions.  There is NO\n"
 		  "warranty; not even for MERCHANTABILITY or FITNESS FOR "
@@ -158,6 +159,11 @@ static int init_curses(void)
 		return 0;
 
 	assert(refresh() != ERR);
+	/* Since refresh() didn't return an error we'll clear errno,
+	 * just in case, since weird errnos returned by refresh() has
+	 * been observed
+	 */
+	errno = 0;
 
 #if NCURSES_MOUSE_VERSION == 1
 	mousemask(BUTTON1_CLICKED, &oldmmask);
@@ -217,6 +223,11 @@ static void update_screen(void)
 	if (continuous) {
 		clrtobot();
 		assert(refresh() != ERR);
+		/* Since refresh() didn't return an error we'll clear errno,
+		 * just in case, since weird errnos returned by refresh() has
+		 * been observed
+		 */
+		errno = 0;
 		assert(move(0, 0) != ERR);
 	}
 #else
@@ -1714,6 +1725,7 @@ int main(int argc, char *argv[])
 {
 	int optc;
 	int opt_index;
+	int tmp_errno;
 	size_t i, start, end;
 	time_t oldtime, now;
 	int maxlen = 0;
@@ -1914,25 +1926,31 @@ int main(int argc, char *argv[])
 
 		/* Update mailstats every mailcheck seconds */
 		if ((eventcheck() && !quit) || now - oldtime > mailcheck) {
-			update_mailbox_stats();
+			if (update_mailbox_stats())
+				break;
 
-			show_mailbox_stats(now, maxlen);
+			if (show_mailbox_stats(now, maxlen))
+				break;
 
 			if (time(&oldtime) == (time_t)-1)
 				break;
 		}
 
 		/* Check for a keypress every second */
-		if (!quit)
+		if (!quit && !errno)
 			(void)sleep((unsigned int)1);
 
-		if (errno == EINTR)
+		if (errno == EINTR || errno == ESRCH)
 			errno = 0;
 	} while (!errno && continuous && !quit);
 
+	tmp_errno = errno;
+
 	if (continuous)
 		reset_curses();
 
+	errno = tmp_errno;
+
 	if (errno == EFBIG)
 		fprintf(stderr,
 			_("%s: %s contains a line longer than %d characters. "
